#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/core.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 02:20
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Third-party packages ###
from click import group

### Local modules ###
from aesir.commands import build, clean, deploy, flush, mine, nodekeys, ping_pong, pull


@group
def cli() -> None:
  """aesir"""


cli.add_command(build, "build")
cli.add_command(clean, "clean")
cli.add_command(deploy, "deploy")
cli.add_command(flush, "flush")
cli.add_command(mine, "mine")
cli.add_command(nodekeys, "nodekeys")
cli.add_command(ping_pong, "ping-pong")
cli.add_command(pull, "pull")


if __name__ == "__main__":
  cli()



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/commands/__init__.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION: https://www.w3docs.com/snippets/python/what-is-init-py-for.html
#
# HISTORY:
# *************************************************************

### Local modules ###
from aesir.commands.build import build
from aesir.commands.clean import clean
from aesir.commands.deploy import deploy
from aesir.commands.flush import flush
from aesir.commands.mine import mine
from aesir.commands.nodekeys import nodekeys
from aesir.commands.ping_pong import ping_pong
from aesir.commands.pull import pull

__all__ = ["build", "clean", "deploy", "flush", "mine", "nodekeys", "ping_pong", "pull"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/commands/flush.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 06:24
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import List, Set

### Third-party packages ###
from click import command
from docker import DockerClient, from_env
from docker.errors import DockerException
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from aesir.configs import DEPRECATED


@command
def flush() -> None:
  """Remove images deprecated by workspace."""
  client: DockerClient
  try:
    client = from_env()
    if not client.ping():
      raise DockerException
  except DockerException:
    rich_print("[red bold]Unable to connect to docker daemon.")
    return
  outputs: List[str] = []
  docker_images: Set[str] = {image.tags[0] for image in client.images.list()}
  for registry_id in track(DEPRECATED, "Remove deprecated images:".ljust(42)):
    if registry_id in docker_images:
      client.images.remove(registry_id)
      outputs.append(f"<Image: '{ registry_id }'> removed.")
  list(map(rich_print, outputs))


__all__ = ["flush"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/types/service.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import List, Literal

### Third-party packages ###
from pydantic import BaseModel, StrictStr

### Local modules ###
from aesir.types.image import ImageAlias


class Service(BaseModel):
  alias: ImageAlias
  command: List[StrictStr] = []
  env_vars: List[StrictStr] = []
  ports: List[StrictStr]


ServiceName = Literal[
  "aesir-bitcoind",
  "aesir-cashu-mint",
  "aesir-lnd",
  "aesir-lnd-krub",
  "aesir-ord",
  "aesir-ping",
  "aesir-pong",
  "aesir-postgres",
  "aesir-redis",
]

__all__ = ["Service", "ServiceName"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/types/image.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import Literal

ImageAlias = Literal["bitcoind", "cashu-mint", "lnd", "lnd-krub", "ord", "postgres", "redis"]
ImageEnum = Literal["optional", "required"]

__all__ = ["ImageAlias", "ImageEnum"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/configs.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from pathlib import Path
from typing import Any, Dict, List, Optional

### Standard packages ###
from pydantic import TypeAdapter
from yaml import Loader, load

### Local modules ###
from aesir.types import (
  Build,
  BuildEnum,
  ClusterEnum,
  ImageAlias,
  ImageEnum,
  PeripheralEnum,
  Service,
  ServiceName,
)

BUILDS: Dict[BuildEnum, Build]
CLUSTERS: Dict[ClusterEnum, Dict[ServiceName, Service]]
DEPRECATED: List[str]
IMAGES: Dict[ImageEnum, Dict[ImageAlias, str]]
NETWORK: str
PERIPHERALS: Dict[PeripheralEnum, Dict[ServiceName, Service]]

file_path: Path = Path(__file__).resolve()
with open(str(file_path).replace("configs.py", "schemas.yml"), "rb") as stream:
  schema: Optional[Dict[str, Any]] = load(stream, Loader=Loader)
  if schema:
    BUILDS = TypeAdapter(Dict[BuildEnum, Build]).validate_python(schema["builds"])
    CLUSTERS = TypeAdapter(Dict[ClusterEnum, Dict[ServiceName, Service]]).validate_python(
      schema["clusters"]
    )
    DEPRECATED = schema.get("deprecated", [])
    IMAGES = TypeAdapter(Dict[ImageEnum, Dict[ImageAlias, str]]).validate_python(schema["images"])
    NETWORK = schema.get("network", "aesir")
    PERIPHERALS = TypeAdapter(Dict[PeripheralEnum, Dict[ServiceName, Service]]).validate_python(
      schema["peripherals"]
    )

__all__ = ["BUILDS", "CLUSTERS", "DEPRECATED", "IMAGES", "NETWORK", "PERIPHERALS"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/types/__init__.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION: https://www.w3docs.com/snippets/python/what-is-init-py-for.html
#
# HISTORY:
# *************************************************************

### Local modules ###
from aesir.types.blockchain_info import BlockchainInfo
from aesir.types.build import Build, BuildEnum
from aesir.types.cluster_enum import ClusterEnum
from aesir.types.image import ImageAlias, ImageEnum
from aesir.types.lnd_info import LNDInfo
from aesir.types.mutex_option import MutexOption
from aesir.types.new_address import NewAddress
from aesir.types.open_channel import OpenChannel
from aesir.types.peripheral_enum import PeripheralEnum
from aesir.types.service import Service, ServiceName


__all__ = [
  "BlockchainInfo",
  "Build",
  "BuildEnum",
  "ClusterEnum",
  "ImageAlias",
  "ImageEnum",
  "LNDInfo",
  "MutexOption",
  "NewAddress",
  "OpenChannel",
  "PeripheralEnum",
  "Service",
  "ServiceName",
]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/commands/ping_pong.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 06:18
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from re import match
from typing import Dict, List

### Third-party packages ###
from click import argument, command
from docker import DockerClient, from_env
from docker.errors import DockerException, NotFound
from docker.models.containers import Container
from pydantic import TypeAdapter, ValidationError
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from aesir.types import LNDInfo, NewAddress, OpenChannel


@command
@argument("channel_size", default=16777215)
def ping_pong(channel_size: int) -> None:
  """For "duo" cluster, create channels between LND nodes."""
  client: DockerClient
  try:
    client = from_env()
    if not client.ping():
      raise DockerException
  except DockerException:
    rich_print("[red bold]Unable to connect to docker daemon.")
    return

  ### Fetch bitcoind container ###
  bitcoind: Container
  try:
    bitcoind = client.containers.get("aesir-bitcoind")
  except NotFound:
    rich_print('[red bold]Unable to find "aesir-bitcoind" container.')
    return

  ### Initiate parameters ###
  nodekeys: Dict[str, str] = {}
  paddles: List[Container] = list(
    filter(
      lambda container: match(r"aesir-ping|aesir-pong", container.name),
      reversed(client.containers.list()),
    )
  )
  treasuries: Dict[str, str] = {}

  ### Fetch nodekeys ###
  for container in track(paddles, "Fetch LND nodekeys:".ljust(42)):
    lnd_info: LNDInfo = TypeAdapter(LNDInfo).validate_json(
      container.exec_run(
        """
        lncli
          --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
          --rpcserver=localhost:10001
          --tlscertpath=/home/lnd/.lnd/tls.cert
        getinfo
        """
      ).output
    )
    nodekeys[container.name] = lnd_info.identity_pubkey
    new_address: NewAddress = TypeAdapter(NewAddress).validate_json(
      container.exec_run(
        """
        lncli
          --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
          --rpcserver=localhost:10001
          --tlscertpath=/home/lnd/.lnd/tls.cert
        newaddress p2wkh
        """
      ).output
    )
    treasuries[container.name] = new_address.address

  ### Open channels ###
  outputs: List[str] = []
  for container in track(paddles, "Open channels:".ljust(42)):
    if container.name == "aesir-ping":
      try:
        open_channel: OpenChannel = TypeAdapter(OpenChannel).validate_json(
          container.exec_run(
            """
            lncli
              --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
              --rpcserver=localhost:10001
              --tlscertpath=/home/lnd/.lnd/tls.cert
            openchannel %d
              --node_key %s
              --connect aesir-pong:9735
            """
            % (channel_size, nodekeys.get("aesir-pong", ""))
          ).output
        )
        outputs.append(
          f"<Channel 'aesir-ping --> aesir-pong' : txid='{ open_channel.funding_txid }'>"
        )
        bitcoind.exec_run(
          """
          bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir generatetoaddress %d %s
          """
          % (6, treasuries.get("aesir-ping", ""))
        )
      except ValidationError:
        outputs.append("[dim yellow1]Unable to open 'aesir-pong --> aesir-ping' channel.")
    elif container.name == "aesir-pong":
      try:
        open_channel: OpenChannel = TypeAdapter(OpenChannel).validate_json(  # type: ignore[no-redef]
          container.exec_run(
            """
            lncli
              --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
              --rpcserver=localhost:10001
              --tlscertpath=/home/lnd/.lnd/tls.cert
            openchannel %d
              --node_key %s
              --connect aesir-ping:9735
            """
            % (channel_size, nodekeys.get("aesir-ping", ""))
          ).output
        )
        outputs.append(
          f"<Channel 'aesir-pong --> aesir-ping' : txid='{ open_channel.funding_txid }'>"
        )
        bitcoind.exec_run(
          """
          bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir generatetoaddress %d %s
          """
          % (6, treasuries.get("aesir-pong", ""))
        )
      except ValidationError:
        outputs.append("[dim yellow1]Unable to open 'aesir-pong --> aesir-ping' channel.")
  list(map(rich_print, outputs))


__all__ = ["ping_pong"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/__init__.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 05:31
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION: https://www.w3docs.com/snippets/python/what-is-init-py-for.html
#
# HISTORY:
# *************************************************************

### Local modules ###
from aesir.commands import build, clean, deploy, flush, mine, nodekeys, ping_pong, pull
from aesir.types import (
  BlockchainInfo,
  Build,
  BuildEnum,
  ClusterEnum,
  ImageAlias,
  ImageEnum,
  LNDInfo,
  MutexOption,
  NewAddress,
  OpenChannel,
  Service,
  ServiceName,
)


__all__ = [
  "BlockchainInfo",
  "Build",
  "BuildEnum",
  "ClusterEnum",
  "ImageAlias",
  "ImageEnum",
  "LNDInfo",
  "MutexOption",
  "NewAddress",
  "OpenChannel",
  "Service",
  "ServiceName",
  "build",
  "clean",
  "deploy",
  "flush",
  "mine",
  "nodekeys",
  "ping_pong",
  "pull",
]

__version__ = "0.3.7"



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/commands/build.py
# VERSION: 	   0.3.7
# CREATED: 	   2024-02-27 23:52
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from io import BytesIO
from typing import Dict, List, Set

### Third-party packages ###
from click import command, option
from docker import DockerClient, from_env
from docker.errors import BuildError, DockerException
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from aesir.configs import BUILDS
from aesir.types import Build


@command
@option("--cashu-mint", is_flag=True, help="Build cashu-mint optional image", type=bool)
@option("--lnd-krub", is_flag=True, help="Build lnd-krub optional image", type=bool)
@option("--ord", is_flag=True, help="Build ord optional image", type=bool)
@option("--tesla-ball", is_flag=True, help="Build tesla-ball optional image", type=bool)
def build(cashu_mint: bool, lnd_krub: bool, ord: bool, tesla_ball: bool) -> None:
  """Build peripheral images for the desired cluster."""
  client: DockerClient
  try:
    client = from_env()
    if not client.ping():
      raise DockerException
  except DockerException:
    rich_print("[red bold]Unable to connect to docker daemon.")
    return

  ### Build optional images ###
  image_names: List[str] = list(
    map(
      lambda image: image.tags[0].split(":")[0],
      filter(lambda image: len(image.tags) != 0, client.images.list()),
    )
  )
  build_select: Dict[str, bool] = {
    "cashu-mint": cashu_mint,
    "lnd-krub": lnd_krub,
    "ord": ord,
    "tesla-ball": tesla_ball,
  }

  outputs: List[str] = []
  built: Set[str] = {tag for tag in BUILDS.keys() if build_select[tag] and tag in image_names}
  outputs += map(lambda tag: f"<Image: '{tag}'> already exists in local docker images.", built)
  list(map(rich_print, outputs))
  outputs = []

  builds: Dict[str, Build] = {
    tag: build for tag, build in BUILDS.items() if build_select[tag] and tag not in image_names
  }
  if len(builds.keys()) != 0:
    for tag, build in track(builds.items(), description="Build optional images:".ljust(42)):
      with BytesIO("\n".join(build.instructions).encode("utf-8")) as fileobj:
        try:
          client.images.build(fileobj=fileobj, platform=build.platform, tag=tag)
        except BuildError:
          outputs.append(f"[red bold]Build unsuccessful for <Image '{ tag }'>.")
    list(map(rich_print, outputs))


__all__ = ["build"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/commands/pull.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 06:18
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from typing import Dict, List, Set

### Third-party packages ###
from click import command, option
from docker import DockerClient, from_env
from docker.errors import DockerException
from rich import print as rich_print
from rich.progress import track

### Local modules ###
from aesir.configs import IMAGES


@command
@option("--postgres", is_flag=True, help="Pull postgres optional image", type=bool)
@option("--redis", is_flag=True, help="Pull redis optional image", type=bool)
def pull(postgres: bool, redis: bool) -> None:
  """Download required and flagged optional docker images from hub."""
  client: DockerClient
  try:
    client = from_env()
    if not client.ping():
      raise DockerException
  except DockerException:
    rich_print("[red bold]Unable to connect to docker daemon.")
    return

  ### Pull required images ###
  outputs: List[str] = []
  docker_images: Set[str] = {image.tags[0] for image in client.images.list()}
  for registry_id in track(IMAGES["required"].values(), "Pull required images:".ljust(42)):
    if registry_id in docker_images:
      outputs.append(f"<Image: '{ registry_id }'> already exists in local docker images.")
    else:
      repository, tag = registry_id.split(":")
      client.images.pull(repository=repository, tag=tag)
      outputs.append(f"<Image: '{ registry_id }'> downloaded.")
  list(map(rich_print, outputs))

  ### Pull optional images ###
  optional_select: Dict[str, bool] = {"postgres": postgres, "redis": redis}
  optionals: List[str] = [
    registry for alias, registry in IMAGES["optional"].items() if optional_select[alias]
  ]
  if len(optionals) != 0:
    outputs = []
    for registry_id in track(optionals, "Pull optional images flagged:".ljust(42)):
      if registry_id in docker_images:
        outputs.append(f"<Image: '{ registry_id }'> already exists in local docker images.")
      else:
        repository, tag = registry_id.split(":")
        client.images.pull(repository=repository, tag=tag)
        outputs.append(f"<Image: '{ registry_id }'> downloaded.")
    list(map(rich_print, outputs))


__all__ = ["pull"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/commands/mine.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-01 05:31
# AUTHOR:      Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Standard packages ###
from re import match
from typing import List

### Third-party packages ###
from apscheduler.schedulers.background import BackgroundScheduler
from click import argument, command
from docker import DockerClient, from_env
from docker.errors import DockerException, NotFound
from docker.models.containers import Container
from pydantic import TypeAdapter
from rich import print as rich_print
from rich.layout import Layout
from rich.live import Live
from rich.panel import Panel
from rich.progress import track
from rich.table import Table
from rich.text import Text

### Local modules ###
from aesir.types import BlockchainInfo, LNDInfo, NewAddress


@command
@argument("blockcount", default=1, type=int)
@argument("blocktime", default=5, type=int)
def mine(blockcount: int, blocktime: int) -> None:
  """Scheduled mining with "blockcount" and "blocktime"."""
  client: DockerClient
  try:
    client = from_env()
    if not client.ping():
      raise DockerException
  except DockerException:
    rich_print("[red bold]Unable to connect to daemon.")
    return

  ### Retrieve bitcoind container ###
  bitcoind: Container
  try:
    bitcoind = client.containers.get("aesir-bitcoind")
  except NotFound:
    rich_print('[red bold]Unable to find "aesir-bitcoind" container.')
    return

  ### Retrieve other containers ###
  aesir_containers: List[Container] = list(
    filter(lambda container: match(r"aesir-*", container.name), reversed(client.containers.list()))
  )
  container_names: List[str] = list(map(lambda container: container.name, aesir_containers))
  lnd_containers: List[Container] = list(
    filter(
      lambda container: match(r"aesir-lnd|aesir-ping|aesir-pong", container.name),
      aesir_containers,
    )
  )

  ### Generate treasury addresses as mining destinations ###
  treasuries: List[str] = []
  for container in track(lnd_containers, "Generate mining treasuries:".ljust(42)):
    new_address: NewAddress = TypeAdapter(NewAddress).validate_json(
      container.exec_run(
        """
        lncli
          --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
          --rpcserver=localhost:10001
          --tlscertpath=/home/lnd/.lnd/tls.cert
        newaddress p2wkh
        """
      ).output
    )
    treasuries.append(new_address.address)

  ### Set up mining schedule using command arguments ###
  scheduler: BackgroundScheduler = BackgroundScheduler()
  for address in treasuries:
    scheduler.add_job(
      bitcoind.exec_run,
      "interval",
      [
        """
        bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir generatetoaddress %d %s
        """
        % (blockcount, address)
      ],
      seconds=blocktime,
    )
  scheduler.start()

  ### Set up dashboard layout ###
  pane: Layout = Layout()
  sidebar: Layout = Layout(size=24)
  containers: Layout = Layout(name="containers", size=20)
  main: Layout = Layout(size=72)
  body: Layout = Layout(name="body", minimum_size=4, ratio=8, size=17)
  footer: Layout = Layout(name="footer", size=3)
  pane.split_row(sidebar, main)
  main.split_column(body, footer)
  sidebar.split_column(containers)

  with Live(pane, refresh_per_second=4, transient=True):
    pane["containers"].update(Panel(Text("\n".join(container_names)), title="containers"))
    while True:
      ### Update ###
      blockchain_info: BlockchainInfo = TypeAdapter(BlockchainInfo).validate_json(
        bitcoind.exec_run(
          """
          bitcoin-cli -regtest -rpcuser=aesir -rpcpassword=aesir getblockchaininfo
          """
        ).output
      )

      ### Initiate parameters ###
      lnd_infos: List[LNDInfo] = []
      lnd_names: List[str] = []

      ## Fetch LNDInfo from LND containers ###
      for container in lnd_containers:
        lnd_info: LNDInfo = TypeAdapter(LNDInfo).validate_json(
          container.exec_run(
            """
            lncli
              --macaroonpath=/home/lnd/.lnd/data/chain/bitcoin/regtest/admin.macaroon
              --rpcserver=localhost:10001
              --tlscertpath=/home/lnd/.lnd/tls.cert
            getinfo
            """
          ).output
        )
        lnd_infos.append(lnd_info)
        lnd_names.append(container.name)

      ### Draw ###
      body_table: Table = Table(expand=True, show_lines=True)
      body_table.add_column("Name", justify="left", style="green")
      body_table.add_column("Nodekey", justify="left", style="cyan")
      body_table.add_column("Channels")
      body_table.add_column("Peers")
      body_table.add_column("Height")
      body_table.add_column("Synced?", justify="right")
      for i, lnd_info in enumerate(lnd_infos):
        body_table.add_row(
          lnd_names[i],
          "\n".join(
            lnd_info.identity_pubkey[c : c + 11]
            for c in range(0, len(lnd_info.identity_pubkey), 11)
          ),
          f"{lnd_info.num_active_channels}",
          f"{lnd_info.num_peers}",
          f"{lnd_info.block_height}",
          ("[red]false", "[green]true")[lnd_info.synced_to_chain],
        )
      pane["body"].update(body_table)
      pane["footer"].update(
        Panel(
          Text.assemble(
            ("Chain: ", "bright_magenta bold"),
            blockchain_info.chain.ljust(9),
            ("Blocks: ", "green bold"),
            f"{blockchain_info.blocks}".ljust(8),
            ("Size: ", "blue bold"),
            f"{blockchain_info.size_on_disk}".ljust(10),
            ("Time: ", "cyan bold"),
            f"{blockchain_info.time}".rjust(10),
          )
        )
      )


__all__ = ["mine"]



#!/usr/bin/env python3.8
# coding:utf-8
# Copyright (C) 2022-2024 All rights reserved.
# FILENAME:    ~~/src/aesir/types/open_channel.py
# VERSION: 	   0.3.7
# CREATED: 	   2023-12-02 20:50
# AUTHOR: 	   Sitt Guruvanich <aekasitt.g+github@siamintech.co.th>
# DESCRIPTION:
#
# HISTORY:
# *************************************************************

### Third-party packages ###
from pydantic import BaseModel, StrictStr


class OpenChannel(BaseModel):
  funding_txid: StrictStr


__all__ = ["OpenChannel"]
